/***
 * Copyright (C) 1997,98 -- Miroslav Zagorac  <zaga@fly.cc.fer.hr>
 */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <string.h>
#include <stdlib.h>
#include <sys/param.h>

#include "lib_debug.h"
#include "lib_font.h"
#include "lib_image.h"


#ifdef ANSI_FUNC

void font_char_color (struct FONT_DATA *font, short color, short char_index)
#else

void font_char_color (font, color, char_index)
   struct FONT_DATA *font;
   short color;
   short char_index;
#endif
{
   register unchar *font_bitmap;
   short            i, char_size;

   debug (("{%s(0x%08lx, %d, %d)", __FUNCTION__, (long) font, color, char_index))

   char_size   = (long) font->width * font->height;
   font_bitmap = font->bitmap + char_index * char_size;

   debug (("char_size   = %ld\n", char_size))
   debug (("font_bitmap = 0x%08x\n", font_bitmap))

   for (i = 0; i < char_size; i++, font_bitmap++)
      if (*font_bitmap)
      {
         /***
          * Ako karakter vec ima potrebnu boju, onda cemo ustediti malo vremena...
          */
         if (*font_bitmap == color)
            break;
         else
         {
            *font_bitmap++ = (unchar) color;

            for ( ; i < char_size; i++, font_bitmap++)
               if (*font_bitmap)
                  *font_bitmap = (unchar) color;
         }
      }

   debug_endfunc
}


#ifdef ANSI_FUNC

void font_color (struct FONT_DATA *font, short color)
#else

void font_color (font, color)
   struct FONT_DATA *font;
   short color;
#endif
{
   register unchar *font_bitmap;
   long             i, font_size;

   debug (("{%s(0x%08lx, %d)", __FUNCTION__, (long) font, color))

   font_size   = (long) font->width * font->height * 256;
   font_bitmap = font->bitmap;

   debug (("font_size   = %ld\n", font_size))
   debug (("font_bitmap = 0x%08x\n", font_bitmap))

   for (i = 0; i < font_size; i++, font_bitmap++)
      if (*font_bitmap)
      {
         if (*font_bitmap == color)
            break;
         else
         {
            *font_bitmap++ = (unchar) color;

            for ( ; i < font_size; i++, font_bitmap++)
               if (*font_bitmap)
                  *font_bitmap = (unchar) color;
         }
      }

   debug_endfunc
}


#ifdef ANSI_FUNC

struct FONT_DATA *font_load (char *directory, char *fontname)
#else

struct FONT_DATA *font_load (directory, file)
   char *directory;
   char *fontname;
#endif
{
   static struct FONT_DATA *font;
   struct IMAGE_DATA       *font_image;
   short                    font_chars, font_charsize, i;
   unchar                  *font_bitmap, *font_bitmap_src;
   char                     filename[MAXPATHLEN];

   debug (("{%s(\"%s\", \"%s\")", __FUNCTION__, directory, fontname))

   if (directory)
      (void) sprintf (filename, "%s/%s", directory, fontname);
   else
      (void) strcpy (filename, fontname);

   if (! (font_image = image_load (filename, NULL, 0)))
      debug_return ("} == 0x%08x", NULL)

   if (! (font = (struct FONT_DATA *) malloc (sizeof (struct FONT_DATA))))
      debug_return ("} == 0x%08x", NULL)

   font->width     = font_image->height;
   font->height    = font_image->height;
   font->writemode = WRITEMODE_MASKED | FONT_EXPANDED;
   font_chars      = font_image->width / font_image->height;
   font_charsize   = font->width * font->height;

   if (! (font->bitmap = (unchar *) calloc (font_chars, font_charsize)))
   {
      free (font);

      debug_return ("} == 0x%08x", NULL)
   }

   font_bitmap     = font->bitmap;
   font_bitmap_src = font_image->image;

   debug (("font_bitmap     = 0x%08x\n", font_bitmap))
   debug (("font_bitmap_src = 0x%08x\n", font_bitmap_src))
   debug (("font_chars      = %d\n", font_chars))
   debug (("font_charsize   = %d\n", font_charsize))

   for (i = 0; i < font_chars; i++)
   {
      short j;

      for (j = 0; j < font->height; j++)
         (void) memcpy (font_bitmap + j * font->width, font_bitmap_src + j * font_image->width, font->width);

      font_bitmap_src += font->width;
      font_bitmap     += font_charsize;
   }

   image_free (&font_image);

   debug_return ("} == 0x%08x", font)
}


#ifdef ANSI_FUNC

void font_free (struct FONT_DATA **font)
#else

void font_free (font)
   struct FONT_DATA **font;
#endif
{
   debug (("{%s(%08lx)", __FUNCTION__, (long) font))

   if (font && *font)
   {
      debug (("Freeing memory at: bitmap=0x%08lx\n", (long) (*font)->bitmap))

      if ((*font)->bitmap)
         free ((*font)->bitmap);

      free (*font);

      *font = NULL;
   }
   else
      debug (("Memory already deleted\n"))

   debug_endfunc
}
